import { EventBus, EventBusOptions, PublishOptions, SubscribeOptions } from "../../interfaces";
import { CloudEvent } from "../../types";
export declare class RabbitMQEventBus<E extends string = string> implements EventBus<E> {
    private connection?;
    private channel?;
    private readonly options;
    private isInitialized;
    private disableReconnection;
    private reconnectAttempts;
    private messageStore;
    private boundQueues;
    private idempotencyStore;
    private connectionHandler;
    private messageHandler?;
    private deadLetterHandler?;
    private readonly MAX_RETRIES;
    private readonly RETRY_DELAYS;
    private readonly RECONNECT_DELAY;
    private readonly MAX_RECONNECT_ATTEMPTS;
    private readonly DLQ_RETENTION;
    private readonly QUEUE_MAX_LENGTH;
    constructor(options: EventBusOptions);
    setDisableReconnection(value: boolean): void;
    private validateState;
    init(): Promise<void>;
    private connect;
    private initializeHandlers;
    private setupExchangesAndQueues;
    private setupEventHandlers;
    private handleUnroutableMessage;
    private republishStoredMessages;
    private hasBindingForRoutingKey;
    publish<T>(event: CloudEvent<T, E>, options?: PublishOptions): Promise<void>;
    subscribe<T>(type: E | E[], handler: (event: CloudEvent<T, E>) => Promise<void>, options?: SubscribeOptions): Promise<void>;
    private reconnect;
    private recreateChannel;
    unsubscribe(consumerTag: string): Promise<void>;
    close(): Promise<void>;
}
