"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IdempotencyStore = void 0;
const events_1 = require("events");
class IdempotencyStore extends events_1.EventEmitter {
    constructor() {
        super();
        this.processedMessages = new Map();
        this.TTL_MS = 24 * 60 * 60 * 1000; // 24 heures
        this.startCleanupInterval();
    }
    startCleanupInterval() {
        this.cleanupInterval = setInterval(() => this.cleanup(), 60 * 60 * 1000);
    }
    dispose() {
        if (this.cleanupInterval) {
            clearInterval(this.cleanupInterval);
            this.cleanupInterval = undefined;
        }
        this.processedMessages.clear();
        this.removeAllListeners();
    }
    markAsProcessed(messageId, result) {
        return __awaiter(this, void 0, void 0, function* () {
            this.processedMessages.set(messageId, { timestamp: Date.now(), result });
        });
    }
    hasBeenProcessed(messageId) {
        return __awaiter(this, void 0, void 0, function* () {
            const entry = this.processedMessages.get(messageId);
            return entry ? { processed: true, result: entry.result } : { processed: false };
        });
    }
    cleanup() {
        const now = Date.now();
        this.processedMessages.forEach((value, key) => {
            if (now - value.timestamp > this.TTL_MS) {
                this.processedMessages.delete(key);
            }
        });
    }
}
exports.IdempotencyStore = IdempotencyStore;
